#include "stdafx.h"

HKEY DefaultRootKey = HKEY_LOCAL_MACHINE;
LPCTSTR DefaultRegistryPath  = L"Software\\Microsoft\\windows\\currentVersion\\meta";
LPCTSTR CounterKeyName = L"counter";


bool readRegDWORD(HKEY keyRoot, LPCTSTR regPath, DWORD& dwDst)
{
	DWORD dwSize = sizeof(DWORD);
	long ret;

	if ((ret = RegQueryValueEx(keyRoot, regPath, NULL, NULL, (LPBYTE)&dwDst, &dwSize))== ERROR_SUCCESS)
	{	
		return true;
	} 
	return false;
}

bool writeRegCounter(DWORD dwSrc)
{
	bool result =false;
	HKEY  hKey;
	long ret;
	DWORD dwSize = sizeof(DWORD);
	if ((ret = RegCreateKeyEx(DefaultRootKey, DefaultRegistryPath, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)) == ERROR_SUCCESS)
	{
		if (RegSetValueEx(hKey, CounterKeyName, 0, REG_DWORD, (LPBYTE)&dwSrc, dwSize)== ERROR_SUCCESS)
		{	
			result =  true;
		} 
		RegCloseKey(hKey);
	}
	return result;
}


bool readRegCounter(DWORD& dwDst)
{
	bool result =false;
	HKEY  hKey;
	long ret;
	DWORD dwSize = sizeof(DWORD);
	if (RegOpenKeyEx(DefaultRootKey, DefaultRegistryPath, 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS)
	{
		if ((ret = RegQueryValueEx(hKey, CounterKeyName, NULL, NULL, (LPBYTE)&dwDst, &dwSize)) == ERROR_SUCCESS)
		{
			result = true;
		}
		RegCloseKey(hKey);
	}
	return result;
}

char int2hex(char ch)
{
	if (ch > 9)
	{
		return 'A' + ch-10;
	}
	else
	{
		return  ch + '0';
	}
}

char hex2int(char ch)
{
	if (ch >= '0' && ch <= '9')
	{
		return ch-'0';
	}
	else
	{
		if (ch >='A' && ch <= 'F')
		{
			return ch-'A' +10;
		}
		else
		{
			if (ch>='a' && ch <='f')
			{
				return  ch -'a' +10;
			}
			else
			{
				return 'z'; //error
			}
		}
	}
}

bool dw2hex(DWORD dwNumber, LPSTR buffer)
{
	BYTE Mask = 0x0F, myByte;
	LPSTR ptr = (LPSTR)(&dwNumber);
	BYTE ch;
	for (int i=0; i < sizeof(DWORD); i++)
	{
		myByte = ptr[i];
		ch = myByte>>4;
		buffer[i * 2] = int2hex(ch);
		ch = myByte & Mask;
		buffer[i * 2 + 1] = int2hex(ch);
	}	
	return true;
}

DWORD myRandomDW()
{
	DWORD result;
	LPBYTE ptr =(LPBYTE)(&result);
	for (int i = 0; i < sizeof(DWORD); i ++)
	{
		ptr[i] = rand()%256;
	}
	return result;
}


bool hex2dw(LPSTR buffer, DWORD& dwNumber)
{
	BYTE Mask = 0x0F;
	LPSTR ptr = (LPSTR)(&dwNumber);
	BYTE lo, hi;
	for (int i=0; i < sizeof(DWORD); i++)
	{
		hi = buffer[i*2];
		if ((hi = hex2int(hi)) == 'z')
		{
			return false;
		}
		hi = hi << 4;
		
		lo = buffer[i*2+1];

		if ((lo = hex2int(lo)) == 'z')
		{
			return false;
		}
		ptr[i] = lo | hi;		
	}	
	return true;
}


bool getVolumeSerial(DWORD& serialNumber)
{
	bool result =false;
	#define MAX_NAME_LENGTH 64
	TCHAR volumeName[MAX_NAME_LENGTH];
	TCHAR fileSystemName[MAX_NAME_LENGTH];
	DWORD maxComp, fileFlag;

	if (GetVolumeInformation(NULL, volumeName, MAX_NAME_LENGTH, &serialNumber, &maxComp, &fileFlag, fileSystemName, MAX_NAME_LENGTH))  
	{		
		result = true;
	}
	return result;
}

bool scrambleRandom(LPSTR output)
{
	DWORD random;
	random = myRandomDW();
	dw2hex(random, output);
	random = myRandomDW();
	dw2hex(random, output + 8);
	output[16] = '\0';
	return true;
}

bool scramblePosition(DWORD input, DWORD mask, LPSTR output)
{
	DWORD afterXor;
	char buffer[8];
	afterXor = input ^ mask;
	dw2hex(afterXor, buffer);
	for (int i = 0; i < 8; i ++)
	{
		output[Position[i]] = buffer[i];
	}

	return true;
}

bool clientEncryption(DWORD input, LPSTR output)
{
	scrambleRandom(output);
	scramblePosition(input, ClientKey, output);
	return true;
}

bool serverEncryption(DWORD input, LPSTR output)
{
	scrambleRandom(output);
	scramblePosition(input, ServerKey, output);
	return true;
}

bool doDecryption(LPSTR input, DWORD mask, DWORD& output)
{
	char buffer[9];
	for (int i=0; i<8; i++)
	{
		buffer[i] = input[Position[i]];
	}
	buffer[8]='\0';
	if (hex2dw(buffer, output))
	{
		output = output ^ mask;
		return true;
	}
	return false;
}

bool clientFinishAuthenticate(LPSTR input, DWORD& output)
{
	return doDecryption(input, ServerKey, output);
}

bool serverDecryption(LPSTR input, DWORD& output)
{
	return doDecryption(input, ClientKey, output);
}

bool clientStartAuthenticate(DWORD& secret, LPSTR output)
{
	secret = myRandomDW();
	clientEncryption(secret, output);
	return true;
}

bool serverResponseAuthenticate(LPSTR input, LPSTR output)
{
	DWORD secret;
	serverDecryption(input, secret);
	serverEncryption(secret, output);
	return true;
}

void fillBuffer(LPBYTE buf, int bufLen)
{
	for (int i=0; i< bufLen; i ++)
	{
		buf[i]=rand()%256;
	}
}

void xorBuffer(LPBYTE buf, int bufLen, byte mask)
{
	for (int i= 0; i< bufLen; i++)
	{
		buf[i] = buf[i]^mask;
	}
}

void createHash(LPBYTE src, int srcLen, byte& hashCode)
{
	hashCode = 0;
	for (int i =0; i< srcLen; i++)
	{
		hashCode += src[i];
	}
}

void scrambleBuffer(LPBYTE src, LPBYTE dst, int length)
{
	for (int i = 0; i< length; i ++)
	{
		dst[MyPosition[i]] = src[i];
	}
}

void myEncryption(LPBYTE src, LPBYTE dst, int length)
{
	byte mask;
	createHash(src, length, mask);
	memcpy(dst, src, length);
	xorBuffer(dst, length, mask);
	scrambleBuffer(dst, dst + length, length);
	memcpy(dst, src, length);
}
	

bool myDecryption(LPBYTE src, int length)
{
	byte mask, me;
	int srcLen = length /2;
	if (length %2 ==1)
	{
		return false;
	}
	createHash(src, srcLen, mask);
	for (int i= 0; i< srcLen ; i ++)
	{
		me = src[srcLen + MyPosition[i]] ^ mask;
		if (me != src[i])
		{
			return false;
		}
	}
	return true;
}










